@@ -1,12 +1,14 @@
-__version__ = "0.7.5"
+__version__ = "0.8.9"
 from .bins import *
 from .perf import *
 from .stats import *
 from .utils import *
 from .props import *
-from .logit import *
-from .regression import *
-from .basics import *
+from .model import *
+from .logistic import *
+from .regress import *
 from .visualize import *
 from .notebook import *
 from .example_data import *
+import pyrsm.basics
+import pyrsm.radiant
