@@ -3,8 +3,10 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.ticker as mtick
 import seaborn as sns
-from pyrsm import xtile, bincode
-from pyrsm.utils import ifelse
+
+# from pyrsm import xtile, bincode
+from .bins import xtile, bincode
+from .utils import ifelse, table2data
 from sklearn import metrics
 from scipy.stats import rankdata
 
@@ -137,7 +139,7 @@ def confusion(df, rvar, lev, pred, cost=1, margin=2):
         Proportion of cases to act on based on the cost/margin ratio
     """
 
-    if isinstance(pred, list) and len(pred) > 1:
+    if pd.api.types.is_list_like(pred) and len(pred) > 1:
         return "This function can only take one predictor variables at time"
 
     break_even = cost / margin
@@ -151,7 +153,7 @@ def confusion(df, rvar, lev, pred, cost=1, margin=2):
     return TP, FP, TN, FN, contact
 
 
-def uplift_tab(df, rvar, lev, pred, tvar, tlev, qnt=10):
+def uplift_tab(df, rvar, lev, pred, tvar, tlev, scale=1, qnt=10):
     """
     Calculate an Uplift table
 
@@ -168,6 +170,8 @@ def uplift_tab(df, rvar, lev, pred, tvar, tlev, qnt=10):
         Name of the treatment variable column in df
     tlev : str
         Name of the 'success' level in tvar
+    scale : float
+        Scaling factor to use in calculations
     qnt : int
         Number of quantiles to create
 
@@ -183,7 +187,9 @@ def uplift_tab(df, rvar, lev, pred, tvar, tlev, qnt=10):
         breaks = np.concatenate(
             (
                 np.array([-np.inf]),
-                np.quantile(x[treatment], np.arange(0, n + 1) / n)[1:-1],
+                np.quantile(x[treatment], np.arange(0, n + 1) / n, method="linear")[
+                    1:-1
+                ],
                 np.array([np.inf]),
             )
         )
@@ -221,10 +227,10 @@ def uplift_tab(df, rvar, lev, pred, tvar, tlev, qnt=10):
             .assign(
                 uplift=lambda x: x.T_resp / x.T_n - x.C_resp / x.C_n,
                 cum_prop=lambda x: x.index / qnt,
-                T_resp=lambda x: x.T_resp.cumsum(),
-                T_n=lambda x: x.T_n.cumsum(),
-                C_resp=lambda x: x.C_resp.cumsum(),
-                C_n=lambda x: x.C_n.cumsum(),
+                T_resp=lambda x: x.T_resp.cumsum() * scale,
+                T_n=lambda x: x.T_n.cumsum() * scale,
+                C_resp=lambda x: x.C_resp.cumsum() * scale,
+                C_n=lambda x: x.C_n.cumsum() * scale,
                 incremental_resp=lambda x: x.T_resp - x.C_resp * x.T_n / x.C_n,
             )
             .assign(
@@ -251,7 +257,9 @@ def uplift_tab(df, rvar, lev, pred, tvar, tlev, qnt=10):
     return tab
 
 
-def inc_uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwargs):
+def inc_uplift_plot(
+    df, rvar, lev, pred, tvar, tlev, scale=1, qnt=10, marker="o", **kwargs
+):
     """
     Plot an Incremental Uplift chart
 
@@ -268,6 +276,8 @@ def inc_uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwarg
         Name of the treatment variable column in df
     tlev : str
         Name of the 'success' level in tvar
+    scale : float
+        Scaling factor to use in calculations
     qnt : int
         Number of quantiles to create
     **kwargs : Named arguments to be passed to the seaborn lineplot function
@@ -277,17 +287,19 @@ def inc_uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwarg
     Seaborn object
         Plot of Incremental Uplift per quantile
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "pred", None)
 
     rd = [
         pd.concat(
             [
                 pd.DataFrame({"cum_prop": [0], "inc_uplift": [0], "pred": p}),
-                uplift_tab(dct[k], rvar, lev, p, tvar, tlev, qnt=qnt).assign(
-                    predictor=p + ifelse(k == "", k, f" ({k})")
-                )[["cum_prop", "inc_uplift", "pred"]],
+                uplift_tab(
+                    dct[k], rvar, lev, p, tvar, tlev, scale=scale, qnt=qnt
+                ).assign(predictor=p + ifelse(k == "", k, f" ({k})"))[
+                    ["cum_prop", "inc_uplift", "pred"]
+                ],
             ]
         )
         for k in dct.keys()
@@ -303,14 +315,166 @@ def inc_uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwarg
         mtick.FuncFormatter(lambda y, _: "{:.0%}".format(y / 100))
     )
     fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
-
-    fig.set(ylabel="Incremental Uplift", xlabel="Proportion of customers")
+    fig.set(ylabel="Incremental Uplift", xlabel="Percentage of population targeted")
     plt.plot([0, 1], [0, yend], linestyle="--", linewidth=1, color=plt.cm.Blues(0.7))
     if len(dct) > 1 or len(pred) > 1:
         fig.legend(title=None)
     return fig
 
 
+def inc_profit_tab(
+    df,
+    rvar,
+    lev,
+    pred,
+    tvar,
+    tlev,
+    cost=1,
+    margin=2,
+    scale=1,
+    qnt=10,
+):
+    """
+    Tabulate Incremental Profit for Uplift modeling
+
+    Parameters
+    ----------
+    df : Pandas dataframe or a dictionary of dataframes with keys to show multiple curves for different models or data samples
+    rvar : str
+        Name of the response variable column in df
+    lev : str
+        Name of the 'success' level in rvar
+    pred : str
+        Name of the column in df with model predictions
+    tvar : str
+        Name of the treatment variable column in df
+    tlev : str
+        Name of the 'success' level in tvar
+    cost : int
+        Cost of an action
+    margin : int
+        Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
+    qnt : int
+        Number of quantiles to create
+
+    Returns
+    -------
+    Seaborn object
+        Plot of Incremental Uplift per quantile
+    """
+
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
+
+    rd = [
+        pd.concat(
+            [
+                pd.DataFrame(
+                    {"cum_prop": [0], "incremental_resp": [0], "T_n": [0], "pred": p}
+                ),
+                uplift_tab(
+                    dct[k], rvar, lev, p, tvar, tlev, scale=scale, qnt=qnt
+                ).assign(predictor=p + ifelse(k == "", k, f" ({k})"))[
+                    ["cum_prop", "incremental_resp", "T_n", "pred"]
+                ],
+            ]
+        )
+        for k in dct.keys()
+        for p in pred
+    ]
+
+    rd = pd.concat(rd).reset_index(drop=True)
+    return rd.assign(inc_profit=(rd.incremental_resp * margin - rd.T_n * cost))
+
+
+def inc_profit_plot(
+    df,
+    rvar,
+    lev,
+    pred,
+    tvar,
+    tlev,
+    cost=1,
+    margin=2,
+    scale=1,
+    qnt=10,
+    contact=True,
+    marker="o",
+    **kwargs,
+):
+    """
+    Plot an Incremental Profit chart for Uplift modeling
+
+    Parameters
+    ----------
+    df : Pandas dataframe or a dictionary of dataframes with keys to show multiple curves for different models or data samples
+    rvar : str
+        Name of the response variable column in df
+    lev : str
+        Name of the 'success' level in rvar
+    pred : str
+        Name of the column in df with model predictions
+    tvar : str
+        Name of the treatment variable column in df
+    tlev : str
+        Name of the 'success' level in tvar
+    cost : int
+        Cost of an action
+    margin : int
+        Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
+    qnt : int
+        Number of quantiles to create
+    contact : bool
+        Plot a vertical line that shows the optimal contact level.
+    marker : str
+        Marker to use for line plot
+    **kwargs : Named arguments to be passed to the seaborn lineplot function
+
+    Returns
+    -------
+    Seaborn object
+        Plot of Incremental Profit per quantile
+    """
+
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
+    group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "pred", None)
+
+    rd = inc_profit_tab(df, rvar, lev, pred, tvar, tlev, cost, margin, scale, qnt)
+    fig = sns.lineplot(
+        x="cum_prop", y="inc_profit", data=rd, hue=group, marker=marker, **kwargs
+    )
+    fig.set(ylabel="Incremental Profit", xlabel="Percentage of population targeted")
+    fig.yaxis.set_major_formatter(mtick.FuncFormatter(lambda y, _: format(int(y), ",")))
+    fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
+    fig.axhline(1, linestyle="--", linewidth=1)
+    if contact:
+        cnf = [
+            confusion(
+                dct[k].query(f"{tvar} == {tlev}"),
+                rvar,
+                lev,
+                p,
+                cost=cost,
+                margin=margin,
+            )[-1]
+            for k in dct.keys()
+            for p in pred
+        ]
+        [
+            [fig.axvline(l, linestyle="--", linewidth=1, color=sns.color_palette()[i])]
+            for i, l in enumerate(filter(lambda x: x < 1, cnf))
+        ]
+    if len(dct) > 1 or len(pred) > 1:
+        fig.legend(title=None)
+
+    return fig
+
+
 def uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwargs):
     """
     Plot an Uplift bar chart
@@ -337,8 +501,8 @@ def uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwargs):
     Seaborn object
         Plot of Incremental Uplift per quantile
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", plt.cm.Blues(0.7))
 
     rd = [
@@ -354,7 +518,7 @@ def uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwargs):
         fig = sns.barplot(x="cum_prop", y="uplift", data=rd, hue=group, **kwargs)
     else:
         fig = sns.barplot(x="cum_prop", y="uplift", data=rd, color=group, **kwargs)
-    fig.set(ylabel="Uplift", xlabel="Proportion of customers")
+    fig.set(ylabel="Uplift", xlabel="Percentage of population targeted")
     fig.yaxis.set_major_formatter(mtick.FuncFormatter(lambda y, _: "{:.0%}".format(y)))
     # fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
 
@@ -363,7 +527,7 @@ def uplift_plot(df, rvar, lev, pred, tvar, tlev, qnt=10, marker="o", **kwargs):
     return fig
 
 
-def profit_max(df, rvar, lev, pred, cost=1, margin=2):
+def profit_max(df, rvar, lev, pred, cost=1, margin=2, scale=1):
     """
     Calculate the maximum profit using a dataframe as input
 
@@ -380,6 +544,8 @@ def profit_max(df, rvar, lev, pred, cost=1, margin=2):
         Cost of an action
     margin : int
         Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
 
     Returns
     -------
@@ -388,10 +554,10 @@ def profit_max(df, rvar, lev, pred, cost=1, margin=2):
     """
 
     TP, FP, TN, FN, contact = confusion(df, rvar, lev, pred, cost=cost, margin=margin)
-    return margin * TP - cost * (TP + FP)
+    return scale * (margin * TP - cost * (TP + FP))
 
 
-def profit(rvar, pred, lev=1, cost=1, margin=2):
+def profit(rvar, pred, lev=1, cost=1, margin=2, scale=1):
     """
     Calculate the maximum profit using series as input. Provides the same results as profit_max
 
@@ -407,6 +573,8 @@ def profit(rvar, pred, lev=1, cost=1, margin=2):
         Cost of an action
     margin : int
         Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
 
     Returns
     -------
@@ -417,7 +585,7 @@ def profit(rvar, pred, lev=1, cost=1, margin=2):
     break_even = cost / margin
     TP = ((rvar == lev) & (pred > break_even)).sum()
     FP = ((rvar != lev) & (pred > break_even)).sum()
-    return margin * TP - cost * (TP + FP)
+    return scale * (margin * TP - cost * (TP + FP))
 
 
 def ROME_max(df, rvar, lev, pred, cost=1, margin=2):
@@ -482,7 +650,7 @@ def ROME(pred, rvar, lev, cost=1, margin=2):
     return profit / (cost * (TP + FP))
 
 
-def profit_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
+def profit_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2, scale=1):
     """
     Calculate table with profit per quantile
 
@@ -501,6 +669,8 @@ def profit_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
         Cost of an action
     margin : int
         Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
 
     Returns
     -------
@@ -509,7 +679,7 @@ def profit_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
     """
 
     df = calc_qnt(df, rvar, lev, pred, qnt=qnt)
-    df["cum_profit"] = margin * df.cum_resp - cost * df.cum_obs
+    df["cum_profit"] = (margin * df.cum_resp - cost * df.cum_obs) * scale
     df0 = pd.DataFrame({"cum_prop": [0], "cum_profit": [0]})
     df = pd.concat([df0, df], sort=False)
     df.index = range(df.shape[0])
@@ -551,7 +721,17 @@ def ROME_tab(df, rvar, lev, pred, qnt=10, cost=1, margin=2):
 
 
 def profit_plot(
-    df, rvar, lev, pred, qnt=10, cost=1, margin=2, contact=False, marker="o", **kwargs
+    df,
+    rvar,
+    lev,
+    pred,
+    qnt=10,
+    cost=1,
+    margin=2,
+    scale=1,
+    contact=True,
+    marker="o",
+    **kwargs,
 ):
     """
     Plot a profit curve
@@ -571,6 +751,8 @@ def profit_plot(
         Cost of an action
     margin : int
         Benefit of an action if a successful outcome results from the action
+    scale : float
+        Scaling factor to use in calculations
     contact : bool
         Plot a vertical line that shows the optimal contact level. Requires
         that `pred` is a series of probabilities. Values equal to 1 (100% contact)
@@ -591,13 +773,13 @@ def profit_plot(
     dct = {"Training": df.query("training == 1"), "Test": df.query("training == 0")}
     profit_plot(dct, "buyer", "yes", "pred_a", cost=0.5, margin=6)
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", None)
     df = [
-        profit_tab(dct[k], rvar, lev, p, qnt=qnt, cost=cost, margin=margin).assign(
-            predictor=p + ifelse(k == "", k, f" ({k})")
-        )
+        profit_tab(
+            dct[k], rvar, lev, p, qnt=qnt, cost=cost, margin=margin, scale=scale
+        ).assign(predictor=p + ifelse(k == "", k, f" ({k})"))
         for k in dct.keys()
         for p in pred
     ]
@@ -605,7 +787,9 @@ def profit_plot(
     fig = sns.lineplot(
         x="cum_prop", y="cum_profit", data=df, hue=group, marker=marker, **kwargs
     )
-    fig.set(ylabel="Profit", xlabel="Proportion of customers")
+    fig.set(ylabel="Profit", xlabel="Percentage of population targeted")
+    fig.yaxis.set_major_formatter(mtick.FuncFormatter(lambda y, _: format(int(y), ",")))
+    fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
     fig.axhline(1, linestyle="--", linewidth=1)
     if contact:
         cnf = [
@@ -614,7 +798,7 @@ def profit_plot(
             for p in pred
         ]
         prof = [
-            profit_max(dct[k], rvar, lev, p, cost=cost, margin=margin)
+            profit_max(dct[k], rvar, lev, p, cost=cost, margin=margin, scale=scale)
             for k in dct.keys()
             for p in pred
         ]
@@ -636,7 +820,7 @@ def profit_plot(
 
 
 def expected_profit_plot(
-    df, rvar, lev, pred, cost=1, margin=2, contact=False, **kwargs
+    df, rvar, lev, pred, cost=1, margin=2, scale=1, contact=True, **kwargs
 ):
     """
     Plot an expected profit curve
@@ -672,8 +856,8 @@ def expected_profit_plot(
     dct = {"Training": df.query("training == 1"), "Test": df.query("training == 0")}
     expected_profit_plot(dct, "buyer", "yes", "pred_a", cost=0.5, margin=6)
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", None)
 
     def calc_exp_profit(df, pred, cost, margin):
@@ -682,7 +866,7 @@ def expected_profit_plot(
         return pd.DataFrame(
             {
                 "cum_prop": np.arange(1, df.shape[0] + 1) / df.shape[0],
-                "cum_profit": np.cumsum(profit),
+                "cum_profit": np.cumsum(profit) * scale,
             }
         )
 
@@ -695,7 +879,9 @@ def expected_profit_plot(
     ]
     df = pd.concat(df).reset_index(drop=True)
     fig = sns.lineplot(x="cum_prop", y="cum_profit", data=df, hue=group, **kwargs)
-    fig.set(ylabel="Profit", xlabel="Proportion of customers")
+    fig.set(ylabel="Expected Profit", xlabel="Percentage of population targeted")
+    fig.yaxis.set_major_formatter(mtick.FuncFormatter(lambda y, _: format(int(y), ",")))
+    fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
     fig.axhline(1, linestyle="--", linewidth=1)
     if contact:
         cnf = [
@@ -703,18 +889,14 @@ def expected_profit_plot(
             for k in dct.keys()
             for p in pred
         ]
-        prof = [
-            profit_max(dct[k], rvar, lev, p, cost=cost, margin=margin)
-            for k in dct.keys()
-            for p in pred
-        ]
+        eprof = df.groupby("predictor").cum_profit.max()
         [
             [
                 fig.axvline(
                     l, linestyle="--", linewidth=1, color=sns.color_palette()[i]
                 ),
                 fig.axhline(
-                    prof[i], linestyle="--", linewidth=1, color=sns.color_palette()[i]
+                    eprof[i], linestyle="--", linewidth=1, color=sns.color_palette()[i]
                 ),
             ]
             for i, l in enumerate(filter(lambda x: x < 1, cnf))
@@ -760,8 +942,8 @@ def ROME_plot(df, rvar, lev, pred, qnt=10, cost=1, margin=2, marker="o", **kwarg
     dct = {"Training": df.query("training == 1"), "Test": df.query("training == 0")}
     ROME_plot(dct, "buyer", "yes", "pred_a", cost=0.5, margin=6)
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", None)
     rd = [
         ROME_tab(dct[k], rvar, lev, p, qnt=qnt, cost=cost, margin=margin).assign(
@@ -776,7 +958,7 @@ def ROME_plot(df, rvar, lev, pred, qnt=10, cost=1, margin=2, marker="o", **kwarg
     )
     fig.set(
         ylabel="Return on Marketing Expenditures (ROME)",
-        xlabel="Proportion of customers",
+        xlabel="Percentage of population targeted",
     )
     fig.axhline(0, linestyle="--", linewidth=1)
     if len(dct) > 1 or len(pred) > 1:
@@ -819,8 +1001,8 @@ def gains_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
     dct = {"Training": df.query("training == 1"), "Test": df.query("training == 0")}
     gains_plot(dct, "buyer", "yes", "pred_a")
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", None)
     rd = [
         gains_tab(dct[k], rvar, lev, p, qnt=qnt).assign(
@@ -833,8 +1015,10 @@ def gains_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
     fig = sns.lineplot(
         x="cum_prop", y="cum_gains", data=rd, hue=group, marker=marker, **kwargs
     )
-    fig.set(ylabel="Cumulative gains", xlabel="Proportion of customers")
+    fig.set(ylabel="Percentage Buyers", xlabel="Percentage of population targeted")
     plt.plot([0, 1], [0, 1], linestyle="--", linewidth=1, color=plt.cm.Blues(0.7))
+    fig.yaxis.set_major_formatter(mtick.FuncFormatter(lambda y, _: "{:.0%}".format(y)))
+    fig.xaxis.set_major_formatter(mtick.FuncFormatter(lambda x, _: "{:.0%}".format(x)))
     if len(dct) > 1 or len(pred) > 1:
         fig.legend(title=None)
     return fig
@@ -875,8 +1059,8 @@ def lift_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
     lift = {"Training": df.query("training == 1"), "Test": df.query("training == 0")}
     lift_plot(dct, "buyer", "yes", "pred_a")
     """
-    dct = ifelse(type(df) is dict, df, {"": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
     group = ifelse(len(pred) > 1 or len(dct.keys()) > 1, "predictor", None)
     rd = [
         lift_tab(dct[k], rvar, lev, p, qnt=qnt).assign(
@@ -889,14 +1073,14 @@ def lift_plot(df, rvar, lev, pred, qnt=10, marker="o", **kwargs):
     fig = sns.lineplot(
         x="cum_prop", y="cum_lift", data=rd, hue=group, marker=marker, **kwargs
     )
-    fig.set(ylabel="Cumulative lift", xlabel="Proportion of customers")
+    fig.set(ylabel="Cumulative lift", xlabel="Percentage of population targeted")
     fig.axhline(1, linestyle="--", linewidth=1)
     if len(dct) > 1 or len(pred) > 1:
         fig.legend(title=None)
     return fig
 
 
-def evalbin(df, rvar, lev, pred, cost=1, margin=2, dec=3):
+def evalbin(df, rvar, lev, pred, cost=1, margin=2, scale=1, dec=3):
     """
     Evaluate binary classification models. Calculates TP, FP, TN, FN, contact, total,
     TPR, TNR, precision, Fscore, accuracy, profit, ROME, AUC, kappa, and profit index
@@ -922,8 +1106,8 @@ def evalbin(df, rvar, lev, pred, cost=1, margin=2, dec=3):
     --------
     """
 
-    dct = ifelse(type(df) is dict, df, {"All": df})
-    pred = ifelse(type(pred) is list, pred, [pred])
+    dct = ifelse(isinstance(df, dict), df, {"All": df})
+    pred = ifelse(isinstance(pred, str), [pred], pred)
 
     def calculate_metrics(key, dfm, pm):
         TP, FP, TN, FN, contact = confusion(dfm, rvar, lev, pm, cost, margin)
@@ -952,7 +1136,7 @@ def evalbin(df, rvar, lev, pred, cost=1, margin=2, dec=3):
             Fscore=[2 * (precision * TPR) / (precision + TPR)],
             accuracy=[(TP + TN) / total],
             kappa=[metrics.cohen_kappa_score(pos, gtbe)],
-            profit=[profit],
+            profit=[profit * scale],
             index=[0],
             ROME=[profit / (cost * (TP + FP))],
             contact=[contact],
@@ -969,7 +1153,7 @@ def evalbin(df, rvar, lev, pred, cost=1, margin=2, dec=3):
     return result.round(dec)
 
 
-def auc(rvar, pred, lev=1):
+def auc(rvar, pred, lev=1, weights=None):
     """
     Calculate area under the RO curve (AUC)
 
@@ -995,12 +1179,21 @@ def auc(rvar, pred, lev=1):
     auc(dvd.buy, rsm.ifelse(dvd.buy == "yes", 1, 0), "yes")
     """
     rvar = np.array(rvar)
-    if type(rvar[0]) != bool or lev is not None:
+    if not isinstance(rvar[0], bool) or lev is not None:
         rvar = rvar == lev
 
-    n1 = np.sum(np.logical_not(rvar))
-    n2 = np.sum(rvar)
+    if weights is None:
+        rd = np.sum(rankdata(pred)[np.logical_not(rvar)])
+        n1 = np.sum(np.logical_not(rvar))
+        n2 = np.sum(rvar)
+    else:
+        pred_df = table2data(
+            pd.DataFrame({"pred": pred, "rvar": rvar, "weights": weights}), "weights"
+        )
+        rd = np.sum(rankdata(pred_df.pred)[np.logical_not(pred_df.rvar)])
+        n1 = np.sum(weights[np.logical_not(rvar)])
+        n2 = np.sum(weights[rvar])
 
-    U = np.sum(rankdata(pred)[np.logical_not(rvar)]) - n1 * (n1 + 1) / 2
+    U = rd - n1 * (n1 + 1) / 2
     wt = U / n1 / n2
     return ifelse(wt < 0.5, 1 - wt, wt)
