@@ -1,5 +1,4 @@
 import math
-import re
 import matplotlib.pyplot as plt
 import pandas as pd
 import numpy as np
@@ -7,7 +6,7 @@ import seaborn as sns
 import statsmodels as sm
 from sklearn.inspection import permutation_importance
 from .utils import ifelse, intersect, setdiff
-from .regression import sim_prediction
+from .model import sim_prediction, extract_evars, extract_rvar
 from .perf import auc
 
 
@@ -77,31 +76,7 @@ def distr_plot(df: pd.DataFrame, cols: list = None, nint: int = 25, **kwargs):
     elif df.shape[1] % 2 == 1:
         axes[-1, -1].remove()
 
-    plt.show()
-
-
-def scatter(
-    df: pd.DataFrame,
-    col1: str,
-    col2: str,
-    figsize: tuple[int, int] = (10, 10),
-) -> None:
-    _, ax = plt.subplots(figsize=figsize)
-    ax.set_xlabel(col1)
-    ax.set_ylabel(col2)
-    ax.scatter(df[col1], df[col2])
-
-    plt.show()
-
-
-def extract_evars(model, cn):
-    """
-    Extract a list of the names of the explanatory variables in a statsmodels model
-    """
-    pattern = r"\b\w+\b"
-    evars = re.findall(pattern, model.formula)[1:]
-    evars = [v for v in evars if v in cn]
-    return [v for i, v in enumerate(evars) if v not in evars[:i]]
+    # plt.show()
 
 
 def pred_plot_sm(
@@ -111,11 +86,12 @@ def pred_plot_sm(
     excl=[],
     incl_int=[],
     fix=True,
-    hline=True,
+    hline=False,
     nnv=20,
     minq=0.025,
     maxq=0.975,
 ):
+
     """
     Generate prediction plots for statsmodels regression models (OLS and Logistic).
     A faster alternative to PDP plots.
@@ -173,27 +149,30 @@ def pred_plot_sm(
             return False
 
     rvar = model.endog_names
-    if isinstance(hline, bool) and hline == True:
-        hline = df[rvar].mean()
-
-    if not isinstance(hline, float) and not isinstance(hline, int):
-        hline = False
-        min_max = (np.Inf, -np.Inf)
+    if isinstance(hline, bool):
+        if hline:
+            hline = df[rvar].mean()
+            min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
+        else:
+            hline = False
+            min_max = (np.Inf, -np.Inf)
     else:
         min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
 
     if incl is None:
         incl = extract_evars(model, df.columns)
     else:
-        incl = ifelse(isinstance(incl, list), incl, [incl])
+        incl = ifelse(isinstance(incl, str), [incl], incl)
 
-    excl = ifelse(isinstance(excl, list), excl, [excl])
-    incl_int = ifelse(isinstance(incl_int, list), incl_int, [incl_int])
+    excl = ifelse(isinstance(excl, str), [excl], excl)
+    incl_int = ifelse(isinstance(incl_int, str), [incl_int], incl_int)
 
     if len(excl) > 0:
         incl = [i for i in incl if i not in excl]
 
     nr_plots = len(incl) + len(incl_int)
+    if nr_plots == 0:
+        return None
     fig, ax = plt.subplots(
         max(math.ceil(nr_plots / 2), 2), 2, figsize=(10, 2 * max(nr_plots, 4))
     )
@@ -246,7 +225,7 @@ def pred_plot_sm(
         if sum(is_num) == 2:
             plot_data = (
                 pred_dict[v]
-                .pivot(vl[0], vl[1], "prediction")
+                .pivot(index=vl[0], columns=vl[1], values="prediction")
                 .transpose()
                 .sort_values(vl[1], ascending=False)
             )
@@ -293,13 +272,13 @@ def pred_plot_sm(
 def pred_plot_sk(
     fitted,
     df,
-    transformed=None,
     rvar=None,
     incl=None,
     excl=[],
     incl_int=[],
+    transformed=None,
     fix=True,
-    hline=True,
+    hline=False,
     nnv=20,
     minq=0.025,
     maxq=0.975,
@@ -311,10 +290,8 @@ def pred_plot_sk(
     Parameters
     ----------
     fitted : A fitted sklearn model
-    df : Pandas DataFrame with data used for estimatin. Should include categorical variables
-        in their original for (i.e., before using get_dummies)
-    transformed : List of column names that were transformed using Pandas' get_dummies. If
-        None, the function will try to determine which variables might have been transformed
+    df : Pandas DataFrame with data used for estimation. Should include categorical variables
+        in their original form (i.e., before using get_dummies)
     rvar : The column name for the response/target variable
     incl : A list of column names to generate prediction plots for. If None, all
         variables will be plotted
@@ -322,6 +299,8 @@ def pred_plot_sk(
     incl_int : A list is ":" separated column names to plots interaction plots for.
         For example incl_int = ["a:b", "b:c"] would generate interaction plots for
         variables a x b and b x c
+    transformed : List of column names that were transformed using Pandas' get_dummies. If
+        None, the function will try to determine which variables might have been transformed
     fix : Logical or tuple
         Set the desired limited on yhat or have it calculated automatically.
         Set to False to have y-axis limits set by ggplot2 for each plot
@@ -337,15 +316,26 @@ def pred_plot_sk(
         Quantile to use for the maximum value of numeric variables
     """
     # features names used in the sklearn model
-    fn = fitted.feature_names_in_
+    if hasattr(fitted, "feature_names_in_"):
+        fn = fitted.feature_names_in_
+    else:
+        raise Exception(
+            "This function requires a fitted sklearn model with a named features. If you are using one-hot encoding, please use get_dummies on your dataset before fitting the model."
+        )
 
     not_transformed = [c for c in df.columns for f in fn if c == f]
     if transformed is None:
         transformed = list(
-            set([c for c in df.columns for f in fn if c in f and c != f])
+            set(
+                [
+                    c
+                    for c in df.columns
+                    for f in fn
+                    if f"{c}_" in f and c != f and f"{c}_" not in df.columns
+                ]
+            )
         )
 
-    not_transformed = [c for c in df.columns for f in fn if c == f]
     ints = intersect(transformed, not_transformed)
     if len(ints) > 0:
         trn = '", "'.join(transformed)
@@ -371,23 +361,26 @@ def pred_plot_sk(
     if incl is None:
         incl = not_transformed + transformed
     else:
-        incl = ifelse(type(incl) is list, incl, [incl])
+        incl = ifelse(isinstance(incl, str), [incl], incl)
 
     def dummify(df, trs):
         if len(trs) > 0:
-            return pd.concat([pd.get_dummies(df[trs]), df.drop(trs, axis=1)], axis=1)
+            return pd.concat(
+                [pd.get_dummies(df[trs], columns=trs), df.drop(trs, axis=1)], axis=1
+            )
         else:
             return df
 
-    if rvar is not None and isinstance(hline, bool) and hline == True:
-        hline = df[rvar].mean()
-
     # margin to add above and below in plots
     plot_margin = 0.025
 
-    if not isinstance(hline, float) and not isinstance(hline, int):
-        hline = False
-        min_max = (np.Inf, -np.Inf)
+    if isinstance(hline, bool):
+        if hline and rvar is not None:
+            hline = df[rvar].mean()
+            min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
+        else:
+            hline = False
+            min_max = (np.Inf, -np.Inf)
     else:
         min_max = (hline - plot_margin * hline, hline + plot_margin * hline)
 
@@ -404,8 +397,8 @@ def pred_plot_sk(
         else:
             return False
 
-    excl = ifelse(isinstance(excl, list), excl, [excl])
-    incl_int = ifelse(isinstance(incl_int, list), incl_int, [incl_int])
+    excl = ifelse(isinstance(excl, str), [excl], excl)
+    incl_int = ifelse(isinstance(incl_int, str), [incl_int], incl_int)
 
     if len(excl) > 0:
         incl = [i for i in incl if i not in excl]
@@ -521,7 +514,7 @@ def pred_plot_sk(
         ax[-1, -1].remove()
 
 
-def vimp_plot_sm(fitted, df, rep=5):
+def vimp_plot_sm(fitted, df, rep=10, ax=None, ret=False):
     """
     Creates permutation importance plots for models estimated using the
     statsmodels library
@@ -532,13 +525,20 @@ def vimp_plot_sm(fitted, df, rep=5):
     df : Pandas DataFrame with data used for estimation
     rep: int
         The number of times to resample and calculate the permutation importance
+    ax : axis object
+        Add the plot to a the axis object from a matplotlib subplot
+    ret: bool
+        Return the variable importance table as a sorted DataFrame
     """
+    fw = None
     if hasattr(fitted, "model"):
         model = fitted.model
+        if hasattr(model, "_has_freq_weights") and model._has_freq_weights:
+            fw = model.freq_weights
     else:
         return "This function requires a fitted linear or logistic regression"
 
-    rvar = model.endog_names
+    rvar = extract_rvar(model, df.columns)
     evars = extract_evars(model, df.columns)
     df = df[[rvar] + evars].copy().reset_index(drop=True).dropna()
 
@@ -548,14 +548,16 @@ def vimp_plot_sm(fitted, df, rep=5):
         )
 
     def imp_calc_reg(base, pred):
-        return pd.DataFrame({"y": model.endog, "yhat": pred}).corr().iloc[0, 1] ** 2
+        return (
+            base - pd.DataFrame({"y": model.endog, "yhat": pred}).corr().iloc[0, 1] ** 2
+        )
 
     def imp_calc_logit(base, pred):
-        return base - auc(model.endog, pred)
+        return base - auc(model.endog, pred, weights=fw)
 
     # Calculate the baseline performance
     if isinstance(fitted, sm.genmod.generalized_linear_model.GLMResultsWrapper):
-        baseline_fit = auc(model.endog, fitted.predict(df[evars]))
+        baseline_fit = auc(model.endog, fitted.predict(df[evars]), weights=fw)
         imp_calc = imp_calc_logit  # specifying the function to use
         xlab = "Importance (AUC decrease)"
     elif isinstance(fitted, sm.regression.linear_model.RegressionResultsWrapper):
@@ -591,13 +593,20 @@ def vimp_plot_sm(fitted, df, rep=5):
     sorted_idx = pd.DataFrame(importance_values).transpose()
     sorted_idx = sorted_idx.sort_values(0, ascending=True)
     fig = sorted_idx.plot.barh(
-        color="slateblue", legend=None, figsize=(6, max(5, len(sorted_idx) * 0.5))
+        color="slateblue",
+        legend=None,
+        figsize=(6, max(5, len(sorted_idx) * 0.4)),
+        ax=ax,
     )
     plt.xlabel(xlab)
     plt.title("Permutation Importance")
 
+    if ret:
+        sorted_idx.columns = ["Importance"]
+        return sorted_idx[::-1]
 
-def vimp_plot_sk(fitted, X, y, rep=5):
+
+def vimp_plot_sk(fitted, X, y, rep=5, ax=None, ret=False):
     """
     Creates permutation importance plots for models estimated using the
     sklearn library
@@ -609,6 +618,10 @@ def vimp_plot_sk(fitted, X, y, rep=5):
     y : Series with data for the response variable (target) used for estimation
     rep: int
         The number of times to resample and calculate permutation importance
+    ax : axis object
+        Add the plot to a the axis object from a matplotlib subplot
+    ret: bool
+        Return the variable importance table as a sorted DataFrame
     """
 
     if hasattr(fitted, "classes_"):
@@ -623,13 +636,15 @@ def vimp_plot_sk(fitted, X, y, rep=5):
     )
     data = pd.DataFrame(imp.importances.T)
     data.columns = fitted.feature_names_in_
-    order = data.agg("mean").sort_values(ascending=False).index
-    plt.figure(figsize=(6, max(5, data.shape[0] * 1.5)))
-    fig = sns.barplot(
-        x="value",
-        y="variable",
+    sorted_idx = pd.DataFrame(data.mean().sort_values())
+    fig = sorted_idx.plot.barh(
         color="slateblue",
-        errorbar=None,
-        data=pd.melt(data[order]),
+        legend=None,
+        figsize=(6, max(5, len(sorted_idx) * 0.4)),
+        ax=ax,
     )
     fig = fig.set(title="Permutation Importance", xlabel=xlab, ylabel=None)
+
+    if ret:
+        sorted_idx.columns = ["Importance"]
+        return sorted_idx[::-1]
