@@ -1,7 +1,6 @@
 import pandas as pd
 import numpy as np
-from pyrsm.regression import sim_prediction
-from pyrsm.regression import reg_dashboard
+from pyrsm.model import sim_prediction
 
 np.random.seed(1234)
 nr = 100
@@ -20,3 +19,7 @@ df["response"] = np.where((df["x1"] > 0.5) & (df["response_prob"] > 0.5), "yes",
 def test_sim_prediction():
     ret = sim_prediction(df)
     assert ret.loc[0, "x3"] == "b", "Incorrectly generated simulated data"
+
+
+if __name__ == "__main__":
+    test_sim_prediction()
